
#
# COMP 304 2008
#

from Tkinter import *

#####
# The GridWindow class can be used to generate Sudoku grids of
# various sizes.
class GridWindow:
    
    # Creates a sudoko grid of the specified size
    def __init__(self, master, rows, cols, rowOffset=0, colOffset=0):
        
        self.rows = rows
        self.cols = cols
        
        self.master = master
        self.window = Toplevel(master)
        self.window.title("("+str(rowOffset)+","+str(colOffset)+")")

        # bind UI events to callback methods
        self.window.bind("<Button-1>", self.onClick)
        self.window.bind("<Key>", self.onKey)
        self.window.protocol("WM_DELETE_WINDOW", self.onClose)

        # the grid is drawn on a Tkinter Canvas
        self.canvas = Canvas(self.window, width=cols*40+2, height=rows*40+2)
        self.canvas.pack()
               
        # containers of the GridWindow elements 
        self.rectangle = {}
        self.text = {}
        
        # The coordinate of the square that currently has focus. 
        # A square's coordinate is stored as a tuple (row, col),
        # where row and col are integers in the range 0 to 9
        self.focus = None 
        
        # These are the keyboard keys that can be used to clear a
        # square. Right now, these are space, backspace and enter
        self.clearkey = [13, 8, 32]
    
        for row in range(0, self.rows):
            for col in range(0, self.cols):
                self.rectangle[(row, col)] = \
                  self.canvas.create_rectangle(2+40*col, 2+40*row, 42+40*col, 42+40*row, fill="White")   
                self.text[(row, col)] = \
                  self.canvas.create_text(22+40*col, 22+40*row, text=str((row,col)))
                  # show coordinates in cells
                  # note how on a Tkinter Canvas, (x,y) coordinate (0,0) is the top left corner
    
    # Sets a number in the specified square  
    def setNumber(self, location, number):
        itemId = self.text[location]
        self.canvas.itemconfigure(itemId, text=number)
    
    # Sets the color of the specified square
    def setColor(self, location, color):        
        itemId = self.rectangle[location]
        self.canvas.itemconfigure(itemId, fill=color)
    
    # Sets the color of a given square
    def onClick(self, event):        
        row = (event.y - 2) / 40
        col = (event.x - 2) / 40
                
        if row >= 0 and row < self.rows and col >= 0 and col < self.cols:
            if (self.focus != None):
                self.setColor(self.focus, "White")                
            self.focus = (row, col)
            self.setColor(self.focus, "LightGray")
    
    # This function is called when a keyboard key is pressed when
    # this sudoku window has focus.
    def onKey(self, event):
        if self.focus != None:
            if "123456789".find(event.char) != -1:
                newValue = event.char            
            elif event.keycode in self.clearkey:
                newValue = ""                
            else:
                return
            
            # Here, we are directly setting updating what is displayed in the grid.
            self.setNumber(self.focus, newValue)

            # For the assignment you should use the Observer Pattern.
            # This means the state should be kept centrally and
            # any change to the central state should be reflected in all the appropriate views.
    
    # This function is called then a particular window is closed.
    def onClose(self):
        self.window.destroy()
            
class MainWindow(GridWindow):
    
    def __init__(self, master):
        
        GridWindow.__init__(self, master,9,9)
        
        for i in range(0,2):
            self.canvas.create_line(122+120*i, 2, 122+120*i, 361, width=3)
            self.canvas.create_line(2, 122+120*i, 361, 122+120*i, width=3)

#####
# Creates a 1 by 9 Sudoku grid, suitable for displaying a row.
class RowWindow(GridWindow):
    
    def __init__(self, master, rowOffset, colOffset):
        
        GridWindow.__init__(self, master, 1, 9, rowOffset, colOffset)

#####
# Creates a 9 by 1 Sudoku grid, suitable for displaying a column.        
class ColWindow(GridWindow):
    
    def __init__(self, master, rowOffset, colOffset):
        
        GridWindow.__init__(self, master, 9, 1, rowOffset, colOffset)

#####
# Creates a 3 by 3 Sudoku grid, suitable for displaying a block.
class BlockWindow(GridWindow):
    
    def __init__(self, master, rowOffset, colOffset):
        
        GridWindow.__init__(self, master, 3, 3, rowOffset, colOffset)
                
